'use strict';

var _should = require('should');

var _should2 = _interopRequireDefault(_should);

var _Posts = require('../collections/Posts');

var _Posts2 = _interopRequireDefault(_Posts);

var _Authors = require('../collections/Authors');

var _Authors2 = _interopRequireDefault(_Authors);

var _posts = require('../fixtures/posts');

var _posts2 = _interopRequireDefault(_posts);

var _authors = require('../fixtures/authors');

var _authors2 = _interopRequireDefault(_authors);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

// eslint-disable-line

var _firenze = firenze; /* global describe, before, after, it, firenze, firenzeConfig */
/* eslint-disable no-invalid-this */

var Database = _firenze.Database;
var Promise = _firenze.Promise;

describe('Collection', function () {
  before(function (done) {
    this.db = new Database(firenzeConfig);

    this.Posts = (0, _Posts2.default)(this.db);
    this.postsData = _posts2.default;

    this.Authors = (0, _Authors2.default)(this.db);
    this.authorsData = _authors2.default;

    this.db.getAdapter().loadAllFixtures([{
      collection: new this.Posts(),
      rows: this.postsData
    }, {
      collection: new this.Authors(),
      rows: this.authorsData
    }]).then(function () {
      done();
    }).catch(function (error) {
      throw error;
    });
  });

  after(function (done) {
    this.db.close().then(done);
  });

  it('should have an instance', function () {
    var posts = new this.Posts();
    posts.should.have.property('table').which.is.exactly('posts');
  });

  it('should have a model', function () {
    var posts = new this.Posts();
    posts.should.have.property('model');

    var post = posts.model();
    post.should.have.property('get');
  });

  it('should find all results', function (done) {
    var posts = new this.Posts();
    posts.find().all().then(function (models) {
      models.should.be.instanceOf(Array);
      models.should.have.lengthOf(3);

      var firstPost = models[0];
      firstPost.should.have.property('attributes');
      firstPost.attributes.title.should.be.exactly('Hello World');

      done();
    }).catch(function (error) {
      throw error;
    });
  });

  it('should find single result', function (done) {
    var posts = new this.Posts();
    posts.find().where({ id: 1 }).first().then(function (post) {
      post.get('title').should.equal('Hello World');
      done();
    }).catch(function (error) {
      throw error;
    });
  });

  it('should find count of results', function (done) {
    var posts = new this.Posts();
    posts.find().count().run().then(function (count) {
      count.should.equal(3);
      done();
    }).catch(function (error) {
      throw error;
    });
  });

  it('should find single result by primaryKey', function (done) {
    var posts = new this.Posts();
    var promise = posts.findById(1);

    promise.then(function (post) {
      post.get('title').should.equal('Hello World');
      done();
    }).catch(function (error) {
      throw error;
    });
  });

  it('should find single result by field', function (done) {
    var posts = new this.Posts();
    posts.findBy('title', 'Hello World').then(function (post) {
      post.get('title').should.equal('Hello World');
      done();
    }).catch(function (error) {
      throw error;
    });
  });

  it('should fire beforeSave callback', function (done) {
    var posts = new this.Posts({
      beforeSave: function beforeSave(model) {
        return new Promise(function (resolve) {
          model.set('title', 'I am new again');
          return resolve(true);
        });
      }
    });
    var post = posts.model({
      title: 'I am new'
    });

    post.save().then(function (model) {
      model.get('title').should.equal('I am new again');
      done();
    });
  });

  it('should fire afterSave callback', function (done) {
    var posts = new this.Posts({
      afterSave: function afterSave(model) {
        return new Promise(function (resolve) {
          model.set('title', 'I am modified in afterSave');
          return resolve(true);
        });
      }
    });
    var post = posts.model({
      title: 'I am new'
    });

    post.save().then(function (model) {
      model.get('title').should.equal('I am modified in afterSave');
      done();
    });
  });

  it('should fire beforeValidate callback', function (done) {
    var posts = new this.Posts({
      beforeValidate: function beforeValidate(model) {
        return new Promise(function (resolve) {
          model.set('title', 'I am modified in beforeValidate');
          return resolve(true);
        });
      }
    });
    var post = posts.model({
      title: 'I am new'
    });

    post.save().then(function (model) {
      model.get('title').should.equal('I am modified in beforeValidate');
      done();
    });
  });

  it('should fire afterValidate callback', function (done) {
    var posts = new this.Posts({
      afterValidate: function afterValidate(model) {
        return new Promise(function (resolve) {
          model.set('title', 'I am modified in afterValidate');
          return resolve(true);
        });
      }
    });
    var post = posts.model({
      title: 'I am new'
    });

    post.save().then(function (model) {
      model.get('title').should.equal('I am modified in afterValidate');
      done();
    });
  });

  it('should fire beforeDelete callback', function (done) {
    var authors = new this.Authors();
    var author = authors.model({
      id: 1
    });
    author.fetch().then(function (model) {
      model.delete().catch(function (error) {
        error.should.eql(true);
      }).finally(function () {
        done();
      });
    });
  });

  it('should fire afterDelete callback', function (done) {
    var posts = new this.Posts();
    var post = posts.model({
      id: 2
    });
    post.fetch().then(function (model) {
      model.delete().then(function () {
        model.get('_field').should.eql('afterDelete');
        done();
      });
    });
  });
});