'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })(); /* eslint-disable new-cap, no-shadow */

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _validator = require('validator');

var _validator2 = _interopRequireDefault(_validator);

var _getParams = require('get-params');

var _getParams2 = _interopRequireDefault(_getParams);

var _Promise = require('./Promise');

var _Promise2 = _interopRequireDefault(_Promise);

var _Model = require('./Model');

var _Model2 = _interopRequireDefault(_Model);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var Collection = (function () {
  function Collection() {
    var extend = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, Collection);

    this.modelClass = _Model2.default;
    this.table = null;
    this.schema = {};
    this.primaryKey = 'id';
    this.displayField = null;
    this.validationRules = {};
    this.behaviors = [];
    this.loadedBehaviors = [];

    _lodash2.default.merge(this, extend);

    if (!this.alias) {
      this.alias = this.table;
    }

    this.loadBehaviors();
    this.callBehavedMethod(this, 'collectionInitialize');
  }

  _createClass(Collection, [{
    key: 'model',
    value: function model() {
      var attributes = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
      var extend = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      _lodash2.default.merge(extend, {
        collection: this
      });

      return new this.modelClass(attributes, extend);
    }
  }, {
    key: 'getDatabase',
    value: function getDatabase() {
      return this.db;
    }
  }, {
    key: 'setDatabase',
    value: function setDatabase(db) {
      this.db = db;
    }
  }, {
    key: 'getAdapter',
    value: function getAdapter() {
      return this.getDatabase().getAdapter();
    }
  }, {
    key: 'query',
    value: function query() {
      var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      return this.getAdapter().query(_extends({}, options, {
        collection: this
      }));
    }
  }, {
    key: 'find',
    value: function find() {
      return this.query().from(this.table, this.alias);
    }
  }, {
    key: 'findBy',
    value: function findBy(field, value) {
      return this.find().where(_defineProperty({}, field, value)).first();
    }
  }, {
    key: 'findAllBy',
    value: function findAllBy(field, value) {
      return this.find().where(_defineProperty({}, field, value)).all();
    }
  }, {
    key: 'findById',
    value: function findById(value) {
      return this.findBy(this.primaryKey, value);
    }
  }, {
    key: 'findByKey',
    value: function findByKey(value) {
      return this.findById(value);
    }
  }, {
    key: 'validate',
    value: function validate(model) {
      var _this //eslint-disable-line
      = this;

      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var callbacks = _lodash2.default.isUndefined(options.callbacks) || options.callbacks;

      return new _Promise2.default(function (resolve, reject) {
        return _async2.default.waterfall([function (cb) {
          if (!callbacks) {
            return cb(null, true);
          }

          return _this.callBehavedMethod(model, 'beforeValidate').then(function (proceed) {
            return cb(null, proceed);
          }).catch(function (error) {
            return cb(error);
          });
        }, function (proceed, cb) {
          return _this._validate(model).then(function (res) {
            if (res === true) {
              return cb(null, true);
            }

            return cb(res);
          }).catch(function (error) {
            return cb(error);
          });
        }, function (res, cb) {
          if (!callbacks) {
            return cb(null, res);
          }

          return _this.callBehavedMethod(model, 'afterValidate').then(function () {
            return cb(null, res);
          }).catch(function (error) {
            return cb(error);
          });
        }], function (err, result) {
          if (err) {
            if (_lodash2.default.isObject(err)) {
              return resolve(err);
            }

            return reject(err);
          }

          return resolve(result);
        });
      });
    }
  }, {
    key: '_validate',
    value: function _validate(model) {
      var _this2 = this;

      //eslint-disable-line
      var fields = [];
      _lodash2.default.each(model.toObject(), function (v, field) {
        if (!_lodash2.default.isObject(v)) {
          fields.push(field);
        }
      });

      _lodash2.default.each(this.schema, function (schema, field) {
        if (_lodash2.default.isUndefined(schema.validate)) {
          return;
        }

        if (_lodash2.default.isObject(schema.validate) && schema.validate.required) {
          fields.push(field);
          return;
        }

        if (!_lodash2.default.isArray(schema.validate)) {
          return;
        }

        _lodash2.default.each(schema.validate, function (ruleObj) {
          if (ruleObj.required) {
            fields.push(field);
          }
        });
      });

      fields = _lodash2.default.uniq(fields);
      var list = {};

      return new _Promise2.default(function (resolve, reject) {
        _async2.default.mapSeries(fields, function (field, cb) {
          _this2.validateField(model, field).then(function (validated) {
            if (validated !== true) {
              list[field] = validated;
            }

            cb();
          }).catch(function (error) {
            cb(error);
          });
        }, function (err) {
          if (err) {
            return reject(err);
          }

          if (_lodash2.default.isEmpty(list)) {
            return resolve(true);
          }

          return resolve(list);
        });
      });
    }
  }, {
    key: 'validateField',
    value: function validateField(model, field) {
      var _this3 = this;

      var givenValue = arguments.length <= 2 || arguments[2] === undefined ? null : arguments[2];

      var value = !givenValue ? model.get(field) : givenValue;

      var fieldSchema = this.schema[field];
      if (!_lodash2.default.isObject(fieldSchema) || !fieldSchema.validate) {
        return new _Promise2.default.resolve(true); //eslint-disable-line
      }

      var validate = fieldSchema.validate;
      if (!_lodash2.default.isArray(validate)) {
        validate = [validate];
      }

      return new _Promise2.default(function (resolve) {
        _async2.default.eachSeries(validate, function (ruleObj, cb) {
          var rule = ruleObj.rule;
          var ruleName = undefined;
          var ruleOptions = [];
          var message = ruleObj.message;

          var validatorFunc = undefined;
          var validatorOptions = undefined;

          if (_lodash2.default.isString(rule)) {
            ruleName = rule;
          } else if (_lodash2.default.isArray(rule)) {
            ruleName = _lodash2.default.first(rule);
            ruleOptions = _lodash2.default.rest(rule);
          }

          if (_lodash2.default.isFunction(rule)) {
            // rule is a direct function
            validatorFunc = rule;
            validatorOptions = [field, value];
          } else if (ruleName && _lodash2.default.isFunction(_this3.validationRules[ruleName])) {
            // rule is an pre-defined function
            validatorFunc = _this3.validationRules[ruleName];
            validatorOptions = [field, value];
          } else if (_lodash2.default.isFunction(_validator2.default[ruleName])) {
            // validator.js
            validatorFunc = _validator2.default[ruleName];
            validatorOptions = [value].concat(ruleOptions);
          } else {
            // no rule found
            return cb(message);
          }

          var params = (0, _getParams2.default)(validatorFunc);
          if (_lodash2.default.last(params) === 'done') {
            // async
            validatorOptions.push(function (passed) {
              if (!passed) {
                return cb(message);
              }

              cb();
            });
          } else {
            // sync
            var passed = validatorFunc.apply(model, validatorOptions);

            if (!passed) {
              return cb(message);
            }

            cb();
          }
        }, function (err) {
          if (err) {
            return resolve(err);
          }

          return resolve(true);
        });
      });
    }
  }, {
    key: 'save',
    value: function save(model) {
      var _this4 //eslint-disable-line
      //eslint-disable-line
      = this;

      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var callbacks = _lodash2.default.isUndefined(options.callbacks) || options.callbacks;

      return new _Promise2.default(function (resolve, reject) {
        return _async2.default.waterfall([function (cb) {
          if (!callbacks) {
            return cb(null, true);
          }

          return _this4.callBehavedMethod(model, 'beforeSave').then(function (proceed) {
            if (proceed === true) {
              return cb(null, proceed);
            }

            return cb(proceed);
          });
        }, function (proceed, cb) {
          if (!_lodash2.default.isUndefined(options.validate) && options.validate === false) {
            return _this4._save(model, options).then(function (model) {
              return cb(null, model);
            }).catch(function (error) {
              return cb(error);
            });
          }

          return _this4.validate(model).then(function (validated) {
            if (validated === true) {
              return _this4._save(model, options).then(function (model) {
                return cb(null, model);
              }).catch(function (error) {
                return cb(error);
              });
            }

            return cb({
              validationErrors: validated
            });
          });
        }, function (result, cb) {
          if (!callbacks) {
            return cb(null, model);
          }

          return _this4.callBehavedMethod(model, 'afterSave').then(function () {
            return cb(null, model);
          });
        }], function (err, result) {
          if (err) {
            return reject(err);
          }

          return resolve(result);
        });
      });
    }
  }, {
    key: '_save',
    value: function _save(model) {
      var _this5 = this;

      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var obj = model.toObject();
      return new _Promise2.default(function (resolve, reject) {
        var q = null;

        if (model.isNew()) {
          q = _this5.query().create(obj);
        } else {
          obj = _lodash2.default.omit(obj, model.primaryKey);
          if (_lodash2.default.isArray(options.fields)) {
            obj = _lodash2.default.pick(obj, options.fields);
          }

          q = _this5.query().where(_defineProperty({}, _this5.primaryKey, model.getId())).update(obj);
        }

        q.run().then(function (ids) {
          var id = null;
          if (_lodash2.default.isArray(ids) && ids.length === 0 || !ids) {
            return resolve(id);
          } else if (_lodash2.default.isArray(ids)) {
            id = ids[0];
          } else {
            id = ids;
          }

          return _this5.model({ id: id }).fetch().then(function (m) {
            resolve(m);
          }).catch(function (error) {
            reject(error);
          });
        }).catch(reject);
      });
    }
  }, {
    key: 'delete',
    value: function _delete(model) {
      var _this6 //eslint-disable-line
      = this;

      var options = arguments.length <= 1 || arguments[1] === undefined ? {} : arguments[1];

      var callbacks = _lodash2.default.isUndefined(options.callbacks) || options.callbacks;

      return new _Promise2.default(function (resolve, reject) {
        return _async2.default.waterfall([function (cb) {
          if (!callbacks) {
            return cb(null, true);
          }

          return _this6.callBehavedMethod(model, 'beforeDelete').then(function (proceed) {
            return cb(null, proceed);
          }).catch(function (error) {
            return cb(error);
          });
        }, function (proceed, cb) {
          return _this6._delete(model, _this6).then(function (res) {
            return cb(null, res);
          }).catch(function (error) {
            return cb(error);
          });
        }, function (result, cb) {
          return _this6.callBehavedMethod(model, 'afterDelete').then(function () {
            return cb(null, result);
          }).catch(function (error) {
            return cb(error);
          });
        }], function (err, result) {
          if (err) {
            return reject(err);
          }

          return resolve(result);
        });
      });
    }
  }, {
    key: '_delete',
    value: function _delete(model) {
      var _this7 = this;

      return new _Promise2.default(function (resolve, reject) {
        if (model.isNew()) {
          var error = new Error('Cannot delete a model without ID');
          return reject(error);
        }

        _this7.query().delete().where(_defineProperty({}, _this7.primaryKey, model.getId())).run().then(resolve).catch(reject);
      });
    }
  }, {
    key: 'loadBehaviors',
    value: function loadBehaviors() {
      var _this8 = this;

      this.behaviors.forEach(function (behaviorItem) {
        var behaviorClass = behaviorItem;
        var behaviorOptions = {};

        if (_lodash2.default.isObject(behaviorItem) && _lodash2.default.isObject(behaviorItem.options)) {
          behaviorClass = behaviorItem.class;
          behaviorOptions = behaviorItem.options;
        }

        var behavior = new behaviorClass({
          collection: _this8,
          options: behaviorOptions
        });
        _this8.loadedBehaviors.push(behavior);
      });
    }
  }, {
    key: 'callBehavedMethod',
    value: function callBehavedMethod(context, methodName) {
      var _this9 = this;

      if (methodName.indexOf('after') === -1 && methodName.indexOf('before') === -1) {
        // sync
        this.loadedBehaviors.forEach(function (behavior) {
          behavior[methodName](context);
        });

        return true;
      }

      // async
      return new _Promise2.default(function (resolve, reject) {
        return _async2.default.eachSeries(_this9.loadedBehaviors, function (behavior, callback) {
          behavior[methodName](context).then(function (res) {
            return callback(null, res);
          }).catch(function (error) {
            return callback(error);
          });
        }, function (error) {
          if (error) {
            return reject(error);
          }

          return _this9[methodName](context).then(function (res) {
            return resolve(res);
          }).catch(function (error) {
            return reject(error);
          });
        });
      });
    }
  }, {
    key: 'modelInitialize',
    value: function modelInitialize(model) {
      //eslint-disable-line
      return true;
    }
  }, {
    key: 'beforeSave',
    value: function beforeSave(model) {
      //eslint-disable-line
      return new _Promise2.default.resolve(true);
    }
  }, {
    key: 'afterSave',
    value: function afterSave(model) {
      //eslint-disable-line
      return new _Promise2.default.resolve(true);
    }
  }, {
    key: 'beforeValidate',
    value: function beforeValidate(model) {
      //eslint-disable-line
      return new _Promise2.default.resolve(true);
    }
  }, {
    key: 'afterValidate',
    value: function afterValidate(model) {
      //eslint-disable-line
      return new _Promise2.default.resolve(true);
    }
  }, {
    key: 'beforeDelete',
    value: function beforeDelete(model) {
      //eslint-disable-line
      return new _Promise2.default.resolve(true);
    }
  }, {
    key: 'afterDelete',
    value: function afterDelete(model) {
      //eslint-disable-line
      return new _Promise2.default.resolve(true);
    }
  }]);

  return Collection;
})();

exports.default = Collection;