'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _async = require('async');

var _async2 = _interopRequireDefault(_async);

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _Promise = require('./Promise');

var _Promise2 = _interopRequireDefault(_Promise);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var migrationsTableSchema = {
  id: {
    type: 'string',
    nullable: false,
    primary: true
  },
  created: {
    type: 'datetime',
    nullable: true
  }
};

var generatedMigration = _lodash2.default.trimLeft('\nvar Promise = require(\'firenze\').Promise;\n\nmodule.exports = {\n  before: function (db, direction) {\n    return new Promise.resolve(true);\n  },\n\n  up: function (db) {\n    return new Promise.resolve(true);\n  },\n\n  down: function (db) {\n    return new Promise.resolve(true);\n  },\n\n  after: function (db, direction) {\n    return new Promise.resolve(true);\n  }\n};\n');

var Migration = (function () {
  function Migration() {
    var options = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

    _classCallCheck(this, Migration);

    if (typeof options.db === 'undefined') {
      throw new Error('Database instance not provided.');
    }

    this.options = _extends({
      directory: process.cwd() + '/migrations',
      table: 'migrations',
      db: null
    }, options);
  }

  _createClass(Migration, [{
    key: 'initTable',
    value: function initTable() {
      var schema = this.options.db.schema();
      var table = this.options.table;

      return schema.tableExists(table).then(function (exists) {
        if (exists) {
          return true;
        }

        return schema.createTable('migrations', migrationsTableSchema);
      }).catch(function (error) {
        throw error;
      });
    }
  }, {
    key: 'list',
    value: function list() {
      var files = _fs2.default.readdirSync(this.options.directory);
      var _options = this.options;
      var db = _options.db;
      var table = _options.table;

      return new _Promise2.default(function (resolve, reject) {
        db.query().table(table).run().then(function (records) {
          var recordsByIds = {};

          records.forEach(function (record) {
            var id = record.id;

            recordsByIds[id] = record;
          });

          var recordIds = _lodash2.default.keys(recordsByIds);
          var list = [];

          files.forEach(function (file) {
            if (recordIds.indexOf(file) > -1) {
              // already run
              list.push({
                id: file,
                created: recordsByIds[file].created,
                run: true
              });

              return;
            }

            // not run yet
            list.push({
              id: file,
              run: false
            });
          });

          resolve(_lodash2.default.sortByOrder(list, ['id', 'asc']));
        }).catch(function (error) {
          reject(error);
        });
      });
    }
  }, {
    key: 'current',
    value: function current() {
      var _options2 = this.options;
      var db = _options2.db;
      var table = _options2.table;

      return db.query().table(table).orderBy({ created: 'desc' }).first();
    }
  }, {
    key: 'run',
    value: function run(filename) {
      var direction = arguments.length <= 1 || arguments[1] === undefined ? 'up' : arguments[1];

      var allowedDirections = ['up', 'down'];

      if (allowedDirections.indexOf(direction) === -1) {
        throw new Error('Wrong direction provided.');
      }

      var mig = require(this.options.directory + '/' + filename);
      var _options3 = this.options;
      var db = _options3.db;
      var table = _options3.table;

      return new _Promise2.default(function (resolve, reject) {
        _async2.default.waterfall([
        // before
        function (callback) {
          mig.before(db, direction).then(function (response) {
            return callback(null, response);
          }).catch(function (error) {
            return callback(error);
          });
        },

        // up/down
        function (response, callback) {
          mig[direction](db).then(function (response) {
            return callback(null, response);
          }).catch(function (error) {
            return callback(error);
          });
        },

        // after
        function (response, callback) {
          mig.after(db, direction).then(function (response) {
            return callback(null, response);
          }).catch(function (error) {
            return callback(error);
          });
        },

        // create/remove record
        function (response, callback) {
          var query = db.query().table(table);

          if (direction === 'up') {
            query.create({
              id: filename,
              created: new Date()
            });
          } else {
            query.where({
              id: filename
            }).delete();
          }

          query.run().then(function (response) {
            return callback(null, response);
          }).catch(function (error) {
            return callback(error);
          });
        }], function (error, response) {
          if (error) {
            return reject(error);
          }

          resolve(response);
        });
      });
    }
  }, {
    key: 'runAll',
    value: function runAll() {
      var _this = this;

      return new _Promise2.default(function (resolve, reject) {
        _this.list().then(function (list) {
          _async2.default.eachSeries(list, function (item, callback) {
            if (item.run) {
              return callback();
            }

            _this.run(item.id).then(function () {
              callback();
            }).catch(function (error) {
              callback(error);
            });
          }, function (error) {
            if (error) {
              return reject(error);
            }

            resolve(true);
          });
        }).catch(function (error) {
          reject(error);
        });
      });
    }
  }, {
    key: 'generate',
    value: function generate(name) {
      var datetime = new Date().toISOString().slice(0, 19).replace(/[T\-\:]/g, '');
      var filename = datetime + '_' + _lodash2.default.camelCase(name) + '.js';

      var directoryPath = _path2.default.normalize(this.options.directory);
      var fullPath = directoryPath + '/' + filename;

      return new _Promise2.default(function (resolve, reject) {
        _async2.default.waterfall([
        // create directory if needed
        function (callback) {
          _fs2.default.stat(directoryPath, function (error, stats) {
            if (error && error.code !== 'ENOENT') {
              return callback(error);
            }

            if (stats && stats.isDirectory()) {
              return callback(null, true);
            }

            _fs2.default.mkdir(directoryPath, function (error) {
              if (error) {
                return callback('Could not create directory at: ' + directoryPath);
              }

              callback(null, true);
            });
          });
        },

        // create migration file
        function (result, callback) {
          _fs2.default.writeFile(fullPath, generatedMigration, {
            encoding: 'utf8'
          }, function (err) {
            if (err) {
              return callback(err);
            }

            callback(null, fullPath);
          });
        }], function (error, result) {
          if (error) {
            return reject(error);
          }

          resolve(result);
        });
      });
    }
  }]);

  return Migration;
})();

exports.default = Migration;