'use strict';

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _Promise = require('../../Promise');

var _Promise2 = _interopRequireDefault(_Promise);

var _Schema2 = require('../../Schema');

var _Schema3 = _interopRequireDefault(_Schema2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; }

function _createColumns(collectionSchema) {
  return function (t) {
    _lodash2.default.each(collectionSchema, function (column, name) {
      var args = [name];
      var lookForArgs = ['length', 'textType', 'precision', 'scale', 'values'];

      lookForArgs.forEach(function (argName) {
        if (typeof column[argName] !== 'undefined') {
          args.push(column[argName]);
        }
      });

      var c = t[column.type].apply(t, args);

      // primary
      if (column.primary === true) {
        c.primary();
      } else if (_lodash2.default.isArray(column.primary)) {
        c.primary(column.primary);
      }

      // unique
      if (column.unique === true) {
        c.unique();
      }

      // nullable
      if (column.nullable === true) {
        c.nullable();
      } else if (column.nullable === false) {
        c.notNullable();
      }

      // default
      if (typeof column.default !== 'undefined') {
        c.defaultTo(column.default);
      }

      // unsigned
      if (column.unsigned === true) {
        c.unsigned();
      }

      // comment
      if (typeof column.comment !== 'undefined') {
        c.comment(column.comment);
      }
    });
  };
}

var SqlSchema = (function (_Schema) {
  _inherits(SqlSchema, _Schema);

  function SqlSchema() {
    _classCallCheck(this, SqlSchema);

    return _possibleConstructorReturn(this, Object.getPrototypeOf(SqlSchema).apply(this, arguments));
  }

  _createClass(SqlSchema, [{
    key: 'dropTable',
    value: function dropTable(tableName) {
      var connection = this.getConnection();

      return new _Promise2.default(function (resolve, reject) {
        connection.schema.dropTable(tableName).then(function () {
          return resolve.apply(undefined, arguments);
        }).catch(function (error) {
          return reject(error);
        });
      });
    }
  }, {
    key: 'tableExists',
    value: function tableExists(tableName) {
      var connection = this.getConnection();

      return new _Promise2.default(function (resolve, reject) {
        connection.schema.hasTable(tableName).then(function (exists) {
          return resolve(exists);
        }).catch(function (error) {
          return reject(error);
        });
      });
    }
  }, {
    key: 'renameTable',
    value: function renameTable(from, to) {
      var connection = this.getConnection();

      return new _Promise2.default(function (resolve, reject) {
        connection.schema.renameTable(from, to).then(function () {
          return resolve.apply(undefined, arguments);
        }).catch(function (error) {
          return reject(error);
        });
      });
    }
  }, {
    key: 'createTable',
    value: function createTable(tableName, collectionSchema) {
      var connection = this.getConnection();

      return new _Promise2.default(function (resolve, reject) {
        connection.schema.createTable(tableName, _createColumns(collectionSchema)).then(function (results) {
          return resolve(results);
        }).catch(function (error) {
          return reject(error);
        });
      });
    }
  }, {
    key: 'columnExists',
    value: function columnExists(tableName, columnName) {
      var connection = this.getConnection();

      return new _Promise2.default(function (resolve, reject) {
        connection.schema.hasColumn(tableName, columnName).then(function (exists) {
          return resolve(exists);
        }).catch(function (error) {
          return reject(error);
        });
      });
    }
  }, {
    key: 'dropColumns',
    value: function dropColumns(tableName, columnNames) {
      var connection = this.getConnection();

      return new _Promise2.default(function (resolve, reject) {
        connection.schema.dropColumns(tableName, columnNames).then(function (result) {
          return resolve(result);
        }).catch(function (error) {
          return reject(error);
        });
      });
    }
  }, {
    key: 'createColumns',
    value: function createColumns(tableName, collectionSchema) {
      var connection = this.getConnection();

      return new _Promise2.default(function (resolve, reject) {
        connection.schema.table(tableName, _createColumns(collectionSchema)).then(function (results) {
          return resolve(results);
        }).catch(function (error) {
          return reject(error);
        });
      });
    }
  }, {
    key: 'renameColumn',
    value: function renameColumn(tableName, from, to) {
      var connection = this.getConnection();

      return new _Promise2.default(function (resolve, reject) {
        connection.schema.table(tableName, function (t) {
          t.renameColumn(from, to);
        }).then(function (results) {
          return resolve(results);
        }).catch(function (error) {
          return reject(error);
        });
      });
    }

    // @TODO: alterColumn()

  }, {
    key: 'createIndex',
    value: function createIndex(tableName, columns, indexName, indexType) {
      var connection = this.getConnection();

      return new _Promise2.default(function (resolve, reject) {
        connection.schema.table(tableName, function (t) {
          t.index(columns, indexName, indexType);
        }).then(function (results) {
          return resolve(results);
        }).catch(function (error) {
          return reject(error);
        });
      });
    }
  }, {
    key: 'dropIndex',
    value: function dropIndex(tableName, columns, indexName) {
      var connection = this.getConnection();

      return new _Promise2.default(function (resolve, reject) {
        connection.schema.table(tableName, function (t) {
          t.dropIndex(columns, indexName);
        }).then(function (results) {
          return resolve(results);
        }).catch(function (error) {
          return reject(error);
        });
      });
    }
  }]);

  return SqlSchema;
})(_Schema3.default);

exports.default = SqlSchema;