'use strict';

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _createClass = (function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; })();

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = makeQuery;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _Promise = require('../../Promise');

var _Promise2 = _interopRequireDefault(_Promise);

var _Query2 = require('../../Query');

var _Query3 = _interopRequireDefault(_Query2);

var _Expression = require('./Expression');

var _Expression2 = _interopRequireDefault(_Expression);

var _Functions = require('./Functions');

var _Functions2 = _interopRequireDefault(_Functions);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _typeof(obj) { return obj && typeof Symbol !== "undefined" && obj.constructor === Symbol ? "symbol" : typeof obj; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

function _possibleConstructorReturn(self, call) { if (!self) { throw new ReferenceError("this hasn't been initialised - super() hasn't been called"); } return call && (typeof call === "object" || typeof call === "function") ? call : self; }

function _inherits(subClass, superClass) { if (typeof superClass !== "function" && superClass !== null) { throw new TypeError("Super expression must either be null or a function, not " + typeof superClass); } subClass.prototype = Object.create(superClass && superClass.prototype, { constructor: { value: subClass, enumerable: false, writable: true, configurable: true } }); if (superClass) Object.setPrototypeOf ? Object.setPrototypeOf(subClass, superClass) : subClass.__proto__ = superClass; } /* eslint-disable no-invalid-this */

function makeWhere(query, method) {
  for (var _len = arguments.length, args = Array(_len > 2 ? _len - 2 : 0), _key = 2; _key < _len; _key++) {
    args[_key - 2] = arguments[_key];
  }

  if (typeof args[0] !== 'function') {
    var _query$builder;

    (_query$builder = query.builder)[method].apply(_query$builder, args);

    return query;
  }

  query.builder[method](function () {
    var expr = query.expr(this);

    args[0].apply(query, [expr]);
  });

  return query;
}

function extractJoinOptions(table) {
  var alias = table;
  var on = function on() {};

  if (arguments[2] !== 'undefined') {
    alias = arguments[1];
    on = arguments[2];
  } else {
    on = arguments[1];
  }

  return {
    table: table,
    alias: alias,
    on: on
  };
}

function getJoinMethodByType(type) {
  switch (type.toUpperCase()) {
    case 'INNER':
      return 'innerJoin';
    case 'LEFT':
      return 'leftJoin';
    case 'LEFT OUTER':
      return 'leftOuter';
    case 'RIGHT':
      return 'rightJoin';
    case 'RIGHT OUTER':
      return 'rightOuter';
    case 'OUTER':
      return 'outer';
    case 'FULL OUTER':
      return 'fullOuter';
    default:
      return 'leftJoin';
  }
}

function makeQuery(knex) {
  var SqlQuery = (function (_Query) {
    _inherits(SqlQuery, _Query);

    function SqlQuery() {
      var givenOptions = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

      _classCallCheck(this, SqlQuery);

      var options = _extends({
        expressionClass: _Expression2.default,
        functionsClass: _Functions2.default
      }, givenOptions);

      var _this = _possibleConstructorReturn(this, Object.getPrototypeOf(SqlQuery).call(this, options));

      _this.builder = _this.adapter.getConnection().queryBuilder();

      if (_this.collection) {
        _this.table(_this.collection.table);
      }
      return _this;
    }

    _createClass(SqlQuery, [{
      key: 'select',
      value: function select() {
        var _this2 = this;

        for (var _len2 = arguments.length, args = Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
          args[_key2] = arguments[_key2];
        }

        if (args.length === 0) {
          return this;
        }

        args.forEach(function (arg) {
          _this2._select(arg);
        });

        return this;
      }
    }, {
      key: '_select',
      value: function _select(field) {
        var _this3 = this;

        if (typeof field === 'string') {
          this.builder.select(field);
        } else if (_lodash2.default.isArray(field)) {
          field.forEach(function (f) {
            _this3.builder.select(f);
          });
        } else if ((typeof field === 'undefined' ? 'undefined' : _typeof(field)) === 'object') {
          _lodash2.default.each(field, function (f, as) {
            if ((typeof f === 'undefined' ? 'undefined' : _typeof(f)) !== 'object' && typeof f !== 'function') {
              _this3.builder.select(f + ' as ' + as);

              return;
            }

            var funcString = undefined;

            if ((typeof f === 'undefined' ? 'undefined' : _typeof(f)) === 'object' && f instanceof _Functions2.default) {
              funcString = f.toString();
            } else if (typeof f === 'function') {
              var func = _this3.func.bind(_this3);
              funcString = f.bind(_this3)(func).toString();
            }

            var raw = knex.raw(funcString + ' as ' + as);
            _this3.builder.select(raw);
          });
        }

        return this;
      }
    }, {
      key: 'distinct',
      value: function distinct() {
        var givenFields = arguments.length <= 0 || arguments[0] === undefined ? [] : arguments[0];

        var fields = _lodash2.default.isString(givenFields) ? [givenFields] : givenFields;

        if (fields.length === 0) {
          return this;
        }

        this.builder.distinct(fields);

        return this;
      }
    }, {
      key: 'from',
      value: function from(table) {
        var alias = arguments.length <= 1 || arguments[1] === undefined ? null : arguments[1];

        var exp = table;
        if (alias) {
          exp += ' as ' + alias;
        }

        this.table(exp);

        return this;
      }
    }, {
      key: 'table',
      value: function table(_table) {
        this.builder.table(_table);

        return this;
      }
    }, {
      key: 'where',
      value: function where() {
        for (var _len3 = arguments.length, args = Array(_len3), _key3 = 0; _key3 < _len3; _key3++) {
          args[_key3] = arguments[_key3];
        }

        return makeWhere.apply(undefined, [this, 'where'].concat(args));
      }
    }, {
      key: 'andWhere',
      value: function andWhere() {
        for (var _len4 = arguments.length, args = Array(_len4), _key4 = 0; _key4 < _len4; _key4++) {
          args[_key4] = arguments[_key4];
        }

        return makeWhere.apply(undefined, [this, 'andWhere'].concat(args));
      }
    }, {
      key: 'orWhere',
      value: function orWhere() {
        for (var _len5 = arguments.length, args = Array(_len5), _key5 = 0; _key5 < _len5; _key5++) {
          args[_key5] = arguments[_key5];
        }

        return makeWhere.apply(undefined, [this, 'orWhere'].concat(args));
      }
    }, {
      key: 'notWhere',
      value: function notWhere() {
        for (var _len6 = arguments.length, args = Array(_len6), _key6 = 0; _key6 < _len6; _key6++) {
          args[_key6] = arguments[_key6];
        }

        return makeWhere.apply(undefined, [this, 'whereNot'].concat(args));
      }
    }, {
      key: 'limit',
      value: function limit(_limit) {
        this._limit = _limit;
        this.builder.limit(_limit);

        return this;
      }
    }, {
      key: 'page',
      value: function page(_page) {
        if (typeof this._limit === 'undefined') {
          return this;
        }

        var offset = (_page - 1) * this._limit;
        this.offset(offset);

        return this;
      }
    }, {
      key: 'offset',
      value: function offset(_offset) {
        this.builder.offset(_offset);

        return this;
      }
    }, {
      key: 'orderBy',
      value: function orderBy(name) {
        var _this4 = this;

        var direction = arguments.length <= 1 || arguments[1] === undefined ? 'asc' : arguments[1];

        if (_lodash2.default.isObject(name)) {
          _lodash2.default.each(name, function (v, k) {
            _this4.orderBy(k, v);
          });

          return this;
        }

        this.builder.orderBy(name, direction);

        return this;
      }
    }, {
      key: 'groupBy',
      value: function groupBy(givenColumns) {
        var columns = _lodash2.default.isString(givenColumns) ? [givenColumns] : givenColumns;

        this.builder.groupBy(columns);

        return this;
      }
    }, {
      key: 'count',
      value: function count() {
        var _builder;

        (_builder = this.builder).count.apply(_builder, arguments);
        this._count = true;

        return this;
      }
    }, {
      key: 'truncate',
      value: function truncate() {
        this.builder.truncate();

        return this;
      }
    }, {
      key: 'create',
      value: function create(row) {
        this.builder.insert(row);

        return this;
      }
    }, {
      key: 'update',
      value: function update(row) {
        this.builder.update(row);

        return this;
      }
    }, {
      key: 'delete',
      value: function _delete() {
        this.builder.delete();

        return this;
      }
    }, {
      key: 'join',
      value: function join() {
        var givenOptions = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];

        var options = _extends({
          type: null,
          table: null,
          alias: null,
          nest: false,
          on: function on() {}
        }, givenOptions);

        var joinMethod = getJoinMethodByType(options.type);
        var tableName = options.alias ? options.table + ' as ' + options.alias : options.table;
        var query = this;

        this.builder[joinMethod](tableName, function () {
          var expr = query.expr(this, { joins: true });
          options.on.apply(query, [expr]);
        });

        if (options.nest) {
          this.builder.options({
            nestTables: true
          });
        }

        return this;
      }
    }, {
      key: 'innerJoin',
      value: function innerJoin() {
        return this.join(_extends({
          type: 'INNER'
        }, extractJoinOptions.apply(undefined, arguments)));
      }
    }, {
      key: 'leftJoin',
      value: function leftJoin() {
        return this.join(_extends({
          type: 'LEFT'
        }, extractJoinOptions.apply(undefined, arguments)));
      }
    }, {
      key: 'leftOuterJoin',
      value: function leftOuterJoin() {
        return this.join(_extends({
          type: 'LEFT OUTER'
        }, extractJoinOptions.apply(undefined, arguments)));
      }
    }, {
      key: 'rightJoin',
      value: function rightJoin() {
        return this.join(_extends({
          type: 'RIGHT'
        }, extractJoinOptions.apply(undefined, arguments)));
      }
    }, {
      key: 'rightOuterJoin',
      value: function rightOuterJoin() {
        return this.join(_extends({
          type: 'RIGHT OUTER'
        }, extractJoinOptions.apply(undefined, arguments)));
      }
    }, {
      key: 'outerJoin',
      value: function outerJoin() {
        return this.join(_extends({
          type: 'OUTER'
        }, extractJoinOptions.apply(undefined, arguments)));
      }
    }, {
      key: 'fullOuterJoin',
      value: function fullOuterJoin() {
        return this.join(_extends({
          type: 'FULL OUTER'
        }, extractJoinOptions.apply(undefined, arguments)));
      }
    }, {
      key: 'run',
      value: function run() {
        var _this5 = this;

        return new _Promise2.default(function (resolve, reject) {
          _this5.builder.then(function (results) {
            if (_this5._count) {
              return resolve(_lodash2.default.values(results[0])[0]);
            }

            return resolve(results);
          }).catch(reject);
        });
      }
    }, {
      key: 'all',
      value: function all() {
        var _this6 = this;

        return new _Promise2.default(function (resolve, reject) {
          _this6.run().then(function (results) {
            resolve(_this6.toModels(results));
          }).catch(reject);
        });
      }
    }, {
      key: 'first',
      value: function first() {
        var _this7 = this;

        this.limit(1);

        return new _Promise2.default(function (resolve, reject) {
          _this7.run().then(function (results) {
            if (results.length > 0) {
              return resolve(_this7.toModel(results[0]));
            }

            resolve(null);
          }).catch(reject);
        });
      }
    }, {
      key: 'debug',
      value: function debug() {
        console.log('query:', this.builder.toString()); // eslint-disable-line

        return this;
      }
    }]);

    return SqlQuery;
  })(_Query3.default);

  return SqlQuery;
}